@@ -4,8 +4,8 @@
  *
  *     G(a,x) = int_x^infty t^{a-1} exp(-t) dt
  *
- *  for a real and x >= 0. [This differs from 'pgamma' of base R in
- *  that negative values of 'a' are admitted.]
+ *  for 'a' real and 'x' >= 0. [This differs from 'pgamma' of base R
+ *  in that negative values of 'a' are admitted.]
  *
  *  Copyright (C) 2016 Vincent Goulet
  *
@@ -47,6 +47,14 @@
 #include "locale.h"
 #include "expint.h"
 
+
+/*
+ *  IMPLEMENTATION OF THE WORKHORSE
+ *
+ *  Adapted from "special functions" material in the GSL.
+ *
+ */
+
 /* Continued fraction which occurs in evaluation
  * of Q(a,x) or Gamma(a,x).
  *
@@ -63,7 +71,7 @@
 double gamma_inc_F_CF(double a, double x)
 {
     const int    nmax  =  5000;
-    const double small =  R_pow_di(DOUBLE_EPS, 3);
+    const double small =  R_pow_di(DBL_EPSILON, 3);
 
     double hn = 1.0;           /* convergent */
     double Cn = 1.0 / small;
@@ -91,7 +99,7 @@ double gamma_inc_F_CF(double a, double x)
 	Dn = 1.0/Dn;
 	delta = Cn * Dn;
 	hn *= delta;
-	if (fabs(delta-1.0) < DOUBLE_EPS)
+	if (fabs(delta-1.0) < DBL_EPSILON)
 	    break;
     }
 
@@ -187,7 +195,7 @@ double gamma_inc_Q_series(double a, double x)
 	{
 	    t *= -x/(n+1.0);
 	    sum += (a+1.0)/(a+n+1.0)*t;
-	    if (fabs(t/sum) < DOUBLE_EPS)
+	    if (fabs(t/sum) < DBL_EPSILON)
 		break;
 	}
     }
@@ -201,7 +209,7 @@ double gamma_inc_Q_series(double a, double x)
 }
 
 /* Adapted from specfun/gamma_inc.c in GSL sources. Note that base R
- * function 'gammafn' and 'pgamma' are used for postive values of
+ * function 'gammafn' and 'pgamma' are used for positive values of
  * 'a'. */
 double gamma_inc(double a, double x)
 {
@@ -254,17 +262,25 @@ double gamma_inc(double a, double x)
 }
 
 
-/* R interface. Inspired from R sources and dpq.c in actuar */
+/*
+ *  R TO C INTERFACE
+ *
+ *  Adapted from src/main/arithmetic.c in R sources and from a similar
+ *  scheme in package actuar. Main difference: everything is in this
+ *  one file.
+ *
+ */
 #define mod_iterate1(n1, n2, i1, i2)            \
         for (i = i1 = i2 = 0; i < n;            \
              i1 = (++i1 == n1) ? 0 : i1,        \
              i2 = (++i2 == n2) ? 0 : i2,        \
              ++i)
 
+/* Function called by .External() */
 SEXP expint_do_gammainc(SEXP args)
 {
     SEXP sx, sa, sy;
-    int i, ix, ia, n, nx, na;
+    R_xlen_t i, ix, ia, n, nx, na;
     double ai, *a, xi, *x, *y;
     Rboolean naflag = FALSE;
 
@@ -273,8 +289,8 @@ SEXP expint_do_gammainc(SEXP args)
     if (!isNumeric(CAR(args)) || !isNumeric(CADR(args)))
         error(_("invalid arguments"));
 
-    na = LENGTH(CAR(args));
-    nx = LENGTH(CADR(args));
+    na = XLENGTH(CAR(args));
+    nx = XLENGTH(CADR(args));
     if ((na == 0) || (nx == 0))
         return(allocVector(REALSXP, 0));
 
@@ -306,15 +322,9 @@ SEXP expint_do_gammainc(SEXP args)
         warning(R_MSG_NA);
 
     if (n == na)
-    {
-        SET_ATTRIB(sy, duplicate(ATTRIB(sa)));
-        SET_OBJECT(sy, OBJECT(sa));
-    }
+        SHALLOW_DUPLICATE_ATTRIB(sy, sa);
     else if (n == nx)
-    {
-        SET_ATTRIB(sy, duplicate(ATTRIB(sx)));
-        SET_OBJECT(sy, OBJECT(sx));
-    }
+        SHALLOW_DUPLICATE_ATTRIB(sy, sx);
 
     UNPROTECT(3);
 
